
/* Tiny Movie Manager (TMM)
 * 2008 by Tobias W. Kjeldsen 
 * feel free to email: tobias@wkjeldsen.dk
 * third party contributors; Calvin Tai - for his Ftp Bean library */
package tmm;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import ftp.*;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.InputStreamReader;
import java.io.InvalidClassException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;

public class Controller {

    private MovieList movielist;
    private FtpBean ftp;
    private FTPSettings ftpsettings;

    public Controller() {
        movielist = importMovies(); //we check for saved movie-database
        ftp = new FtpBean(); //we use the Ftp Bean library for FTP support
        ftpsettings = importFTPSettings(); //we check for FTP saved settings
    }

    public void addMovie(String name, String length, String format, String director, String releaseyear, String numbercds) {
        if (name.equals("") || format.equals("") || director.equals("")) {
            JOptionPane.showMessageDialog(null, "You are missing some input fields!");
        } else {
            try {
                movielist.addMovie(name, Integer.parseInt(length), format, director, Integer.parseInt(releaseyear), Integer.parseInt(numbercds));
                eksportMovies();
                if (ftpsettings.isFtpdynamic()) {
                    eksportFTPMovies();
                }
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(null, "You have entered a wrong numberformat!");
            }
        }
    }

    public void deleteMovie(String id) {
        try {
            JOptionPane.showMessageDialog(null, movielist.deleteMovie(Integer.parseInt(id)));
            eksportMovies();
            if (ftpsettings.isFtpdynamic()) {
                eksportFTPMovies();
            }
        } catch (NumberFormatException ex) {
            JOptionPane.showMessageDialog(null, "You did not enter a number!");
        }
    }

    public String printAllMovies() {
        return "<html>" + movielist.printAllMovies() + "</html>";
    }

    public String searchMovies(String keyword) {
        return "<html>" + movielist.searchMovies(keyword) + "</html>";
    }

    public String getMovie(int movie) {
        return "<html>" + movielist.getMovie(movie) + "</html>";
    }

    public String printStats() {
        return "<html>Your movielist have the combined playtime of:<p><font color=green size=5>" + movielist.calculateTotalPlaytime() + " hours</font><p>Your movielist have the average playtime of:<p><font color=green size=5>" + movielist.calculateAveragePlaytime() + " hour</font>";
    }

    public String latestMovieAdditions() {
        return "<html>" + movielist.latestMovieAdditions() + "</html>";
    }

    public String[] importIMDB(String imdb_id) {
        String[] movieinfo = new String[4];
        try {
            URL url = new URL("http://wkjeldsen.dk/tmm/imdb.php?id=" + imdb_id);
            URLConnection urlc = url.openConnection();
            DataInputStream in = new DataInputStream(urlc.getInputStream()); // To download
            BufferedReader read = new BufferedReader(new InputStreamReader(in));
            int i = 0;
            while (read.ready() && i < 4) {
                movieinfo[i] = read.readLine();
                i++;
            }
        } catch (IOException ex) {
        //NOTHING
        }
        return movieinfo;
    }

    public void setFTPSettings(String serveraddress, String username, String password, String path, boolean ftpdynamic, boolean ftpenabled) {
        ftpsettings.setServeraddress(serveraddress);
        ftpsettings.setUsername(username);
        ftpsettings.setPassword(password);
        ftpsettings.setPath(path);
        ftpsettings.setFtpdynamic(ftpdynamic);
        ftpsettings.setEnabled(ftpenabled);
        eksportFTPSettings();
    }

    public FTPSettings getFTPSettings() {
        return ftpsettings;
    }

    /* used by GUI - jList/jComboBox */
    public String[] listMovies() {
        return movielist.listMovies();
    }

    private FTPSettings importFTPSettings() {
        ObjectInputStream indlæs;
        FTPSettings ftpsettings_import;
        try {
            indlæs = new ObjectInputStream(new BufferedInputStream(new FileInputStream("settings.config")));
            ftpsettings_import = (FTPSettings) indlæs.readObject();
            indlæs.close();
        } catch (Exception ex) {
            ftpsettings_import = new FTPSettings();
            System.out.println(ex);
        }
        return ftpsettings_import;
    }

    private void eksportFTPSettings() {
        ObjectOutputStream skriv;
        try {
            skriv = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream("settings.config")));
            skriv.writeObject(ftpsettings);
            skriv.close();
        } catch (FileNotFoundException ex) {
            System.out.println(ex);
        } catch (IOException ex) {
            System.out.println(ex);
        }
    }

    private MovieList importMovies() {
        ObjectInputStream indlæs;
        MovieList movielist_import;
        try {
            indlæs = new ObjectInputStream(new BufferedInputStream(new FileInputStream("movies.tmmdb")));
            movielist_import = (MovieList) indlæs.readObject();
            indlæs.close();
        } catch (FileNotFoundException ex) {
            movielist_import = new MovieList();
            System.out.println(ex);
        } catch (ClassNotFoundException ex) {
            movielist_import = new MovieList();
            System.out.println(ex);
        } catch (InvalidClassException ex) {
            JOptionPane.showMessageDialog(null, "Movie-database (movies.tmmdb) could not be imported - incompatible database!");
            movielist_import = new MovieList();
            System.out.println(ex);
        } catch (IOException ex) {
            movielist_import = new MovieList();
            System.out.println(ex);
        }
        return movielist_import;
    }

    private void eksportMovies() {
        ObjectOutputStream skriv;
        try {
            skriv = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream("movies.tmmdb")));
            skriv.writeObject(movielist);
            skriv.close();
        } catch (FileNotFoundException ex) {
            System.out.println(ex);
        } catch (IOException ex) {
            System.out.println(ex);
        }
    }

    /* public because UI uses it through an actionperformed event */
    public void eksportFTPMovies() {
        /* Ftp Bean library usage here */
        try {
            if (ftpsettings.isEnabled()) {
                ftp.ftpConnect(ftpsettings.getServeraddress(), ftpsettings.getUsername(), ftpsettings.getPassword());
                ftp.setDirectory(ftpsettings.getPath()); //without starting and ending slashes
                ftp.putBinaryFile("movies.tmmdb", "movies.tmmdb");
                ftp.close();
            } else {
                JOptionPane.showMessageDialog(null, "FTP-support currently disabled; enable under FTP settings");
            }
        } catch (UnknownHostException ex) {
            JOptionPane.showMessageDialog(null, "There was an error while connection to the suplied FTP server; validate your settings");
        } catch (IOException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FtpException ex) {
            JOptionPane.showMessageDialog(null, "There was an error while connection to the suplied FTP server; validate your settings");
        }
    }

    /* public because UI uses it through an actionperformed event */
    public void importFTPMovies() {
        /* Ftp Bean library usage here */
        try {
            if (ftpsettings.isEnabled()) {
                ftp.ftpConnect(ftpsettings.getServeraddress(), ftpsettings.getUsername(), ftpsettings.getPassword());
                ftp.setDirectory(ftpsettings.getPath()); //without starting and ending slashes
                ftp.getBinaryFile("movies.tmmdb", "movies.tmmdb");
                ftp.close();
                JOptionPane.showMessageDialog(null, "Movie-database (movies.tmmdb) was downloaded from suplied FTP");
                movielist = importMovies(); // we instantiate the new downloaded movies.tmmdb
            } else {
                JOptionPane.showMessageDialog(null, "FTP-support currently disabled; enable under FTP settings");
            }
        } catch (UnknownHostException ex) {
            JOptionPane.showMessageDialog(null, "There was an error while connection to the suplied FTP server; validate your settings");
        } catch (IOException ex) {
            Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FtpException ex) {
            JOptionPane.showMessageDialog(null, "There was an error while connection to the suplied FTP server; validate your settings");
        }
    }

}
