
/* Tiny Movie Manager (TMM)
 * 2008 by Tobias W. Kjeldsen 
 * feel free to email: tobias@wkjeldsen.dk
 * third party contributors; Calvin Tai - for his Ftp Bean library */
package tmm;

import java.io.Serializable;
import java.sql.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.LinkedList;
import java.util.StringTokenizer;

public class MovieList implements Serializable {

    private LinkedList<Movie> movielist;
    private int movieid;

    public MovieList() {
        movielist = new LinkedList<Movie>();
    }

    public void addMovie(String name, int length, String format, String director, int releaseyear, int numbercds) {
        /* we generate a unique id on
         * on base of the movieid
         * because our list is Serializable
         * we use this method to secure a unique ID */
        movielist.add(new Movie(movieid++, name, length, format, director, releaseyear, numbercds));
    }

    public String deleteMovie(int id) {
        String temp = "";
        boolean found = false;
        int i = 0;
        while (i < movielist.size() && !found) {
            if (movielist.get(i).getMovieid() == id) {
                temp = movielist.get(i).getName() + " was deleted!";
                movielist.remove(i);
                found = true;
            }
            i++;
        }
        if (!found) {
            temp = "No movie with entered ID found! ID: " + id;
        }
        return temp;
    }

    public String searchMovies(String keyword) {
        String temp = "";
        int i = 0;
        boolean found = false;
        while (i < movielist.size() && !found) {
            StringTokenizer token = new StringTokenizer(movielist.get(i).getName());
            while (token.hasMoreElements()) {
                if (token.nextToken().equalsIgnoreCase(keyword)) {
                    temp = movielist.get(i).toString();
                    found = true;
                }
            }
            i++;
        }
        if (!found) {
            int b = 0;
            while (b < movielist.size() && !found) {
                if (movielist.get(b).getName().equalsIgnoreCase(keyword)) {
                    temp = movielist.get(b).toString();
                    found = true;
                }
                b++;
            }
        }
        return temp;
    }

    public String getMovie(int movie) {
        return movielist.get(movie).toString();
    }

    public String calculateTotalPlaytime() {
        NumberFormat formatter = new DecimalFormat("#.##");
        double totallength = 0;
        for (int i = 0; i < movielist.size(); i++) {
            totallength += movielist.get(i).getLength();
        }
        /* calculates total hours of all movies */
        return formatter.format((totallength * 60) / 3600);
    }

    public String calculateAveragePlaytime() {
        NumberFormat formatter = new DecimalFormat("#.##");
        double totallength = 0;
        int movies = 0;
        for (int i = 0; i < movielist.size(); i++) {
            totallength += movielist.get(i).getLength();
            movies++;
        }
        /* calculates average hours of all movies */
        return formatter.format(((totallength * 60) / 3600) / movies);
    }

    /* this only gets the latest 5 additions to the database */
    public String latestMovieAdditions() {
        String temp = "";
        if (movielist.size() < 4) {
            for (int i = 0; i < movielist.size(); i++) {
                temp += movielist.get(i).toString() + "<br><br>";
            }
        } else {
            for (int i = (movielist.size() - 4); i < movielist.size(); i++) {
                temp += movielist.get(i).toString() + "<br><br>";
            }
        }
        if (movielist.size() == 0) {
            temp = "<font color=red>No movies in current database!</font>";
        }
        return temp;
    }

    /* used by GUI - jList/jComboBox */
    public String[] listMovies() {
        String[] movies = new String[movielist.size()];
        int i = 0;
        while (i < movielist.size()) {
            movies[i] = movielist.get(i).getMovieid() + ". " + movielist.get(i).getName() + " (" + movielist.get(i).getReleaseyear() + ")";
            i++;
        }
        return movies;
    }

    public String printAllMovies() {
        String temp = "";
        int i = 0;
        while (i < movielist.size()) {
            temp += movielist.get(i).toString() + "<br><br>";
            i++;
        }
        if (i == 0) {
            temp = "<font color=red>No movies in current database!</font>";
        }
        return temp;
    }
    
}